package com.aledevice.xmlgenerator;

import cn.hutool.core.io.FileUtil;
import cn.hutool.core.io.IoUtil;
import cn.hutool.core.io.file.FileNameUtil;
import cn.hutool.json.JSONObject;
import cn.hutool.json.JSONUtil;
import cn.hutool.poi.excel.ExcelReader;
import cn.hutool.poi.excel.ExcelUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.boot.SpringApplication;
import org.springframework.boot.autoconfigure.SpringBootApplication;
import org.springframework.boot.context.event.ApplicationReadyEvent;
import org.springframework.context.event.EventListener;
import org.springframework.util.StringUtils;

import java.io.File;
import java.io.IOException;
import java.net.URL;
import java.nio.file.Paths;
import java.util.ArrayList;
import java.util.List;

@Slf4j
@SpringBootApplication
public class XmlGeneratorApplication {
    
    private static final String pathname_config = "config/config.json";
    private static final String pathname_excel = "input.xlsx";
    private static final String pathname_dir_out = "out";
    private static final String pathname_dir_tmp = "tmp";
    private static final String pathname_template = "template.xml";
    private File workspace;
    private String xmlNamePattern;
    private List<List<String>> excelData = new ArrayList<>();
    
    public static void main(String[] args) {
        SpringApplication.run(XmlGeneratorApplication.class, args);
    }
    
    @EventListener(ApplicationReadyEvent.class)
    public void run() {
        log.info("xml generator start");
        try {
            loadProperties();
            loadExcelFile();
            generateXmlFiles();
        } catch (Exception e) {
            log.error("there is something error.", e);
        } finally {
            clean();
            log.info("xml generator finish");
            closeGracefully();
        }
    }
    
    private void clean() {
        FileUtil.clean(new File(workspace, pathname_dir_tmp));
    }
    
    private void closeGracefully() {
        try {
            log.info("Please click anything to close me.");
            System.in.read();
        } catch (IOException e) {
            log.info("not close gracefully", e);
        }
    }
    
    private void generateXmlFiles() {
        File outDir = new File(workspace, pathname_dir_out);
        String templateContent = IoUtil.readUtf8(IoUtil.toStream(new File(workspace, pathname_template)));
        List<String> header = excelData.get(0);
        for (int i = 1; i < excelData.size(); i++) {
            List<String> row = excelData.get(i);
            String xmlFileName = getFileName(header, row);
            File xmlFile = FileUtil.touch(new File(outDir, xmlFileName));
            writeContent(xmlFile, header, row, templateContent);
        }
    }
    
    private void writeContent(File xmlFile, List<String> header, List<String> row, String templateContent) {
        String content = templateContent;
        for (int i = 0; i < header.size(); i++) {
            String key = listGetNullSafe(header, i);
            String value = listGetNullSafe(row, i);
            if (StringUtils.hasText(key)) {
                content = content.replace("$$$" + key + "$$$", value);
            }
        }
        IoUtil.writeUtf8(FileUtil.getOutputStream(xmlFile), true, content);
        log.info("generator xml success. xml filename:{}", xmlFile.getName());
    }
    
    private String getFileName(List<String> header, List<String> row) {
        int index = 0;
        for (int i = 0; i < header.size(); i++) {
            if (header.get(i).equalsIgnoreCase("MAC Address")) {
                index = i;
                break;
            }
        }
        String mac = row.get(index);
        return FileNameUtil.cleanInvalid(xmlNamePattern.replace("$$$mac$$$", mac));
    }
    
    private void loadExcelFile() {
        File excelFile = new File(workspace, pathname_excel);
        ExcelReader excelReader = ExcelUtil.getReader(excelFile);
        List<List<Object>> table = excelReader.read();
        for (List<Object> row : table) {
            List<String> strRow = new ArrayList<>();
            for (Object cell : row) {
                strRow.add(cellToString(cell));
            }
            excelData.add(strRow);
        }
    }
    
    private String cellToString(Object cell) {
        try {
            return cell.toString();
        } catch (Exception e) {
            return "";
        }
    }
    
    private void loadProperties() {
        workspace = getWorkspace();
        File configFile = new File(workspace, pathname_config);
        JSONObject config = JSONUtil.parseObj(IoUtil.readUtf8(IoUtil.toStream(configFile)));
        xmlNamePattern = config.getStr("xmlNamePattern");
        log.info("load properties. workspace: {}, xmlNamePattern: {}", workspace.getPath(), xmlNamePattern);
    }
    
    private File getWorkspace() {
        try {
            String url = XmlGeneratorApplication.class.getResource(XmlGeneratorApplication.class.getSimpleName() + ".class").toString();
            String path = url.replaceAll("^jar:(file:.*[.]jar)!/.*", "$1");
            File jarFile = new File(Paths.get(new URL(path).toURI()).toString());
            return jarFile.getParentFile().getParentFile();
        } catch (Exception e) {
            throw new RuntimeException(e);
        }
    }
    
    private String listGetNullSafe(List<String> list, int index){
        try {
            return list.get(index);
        } catch (Exception e) {
            return "";
        }
    }
    
}
